"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EquityDto = exports.PerpEquity = exports.TokenEquity = exports.Equity = exports.MangoAccountData = exports.HealthType = exports.PerpOoDto = exports.PerpOo = exports.PerpPositionDto = exports.PerpPosition = exports.Serum3PositionDto = exports.Serum3Orders = exports.TokenPositionDto = exports.TokenPosition = exports.MangoAccount = void 0;
const bytes_1 = require("@project-serum/anchor/dist/cjs/utils/bytes");
const market_1 = require("@project-serum/serum/lib/market");
const constants_1 = require("../constants");
const utils_1 = require("../utils");
const healthCache_1 = require("./healthCache");
const I80F48_1 = require("./I80F48");
const serum3_1 = require("./serum3");
class MangoAccount {
    constructor(publicKey, group, owner, name, delegate, beingLiquidated, accountNum, bump, netDeposits, netSettled, headerVersion, tokens, serum3, perps, perpOpenOrders, accountData) {
        this.publicKey = publicKey;
        this.group = group;
        this.owner = owner;
        this.delegate = delegate;
        this.accountNum = accountNum;
        this.accountData = accountData;
        this.name = bytes_1.utf8.decode(new Uint8Array(name)).split('\x00')[0];
        this.tokens = tokens.map((dto) => TokenPosition.from(dto));
        this.serum3 = serum3.map((dto) => Serum3Orders.from(dto));
        this.perps = perps.map((dto) => PerpPosition.from(dto));
        this.perpOpenOrders = perpOpenOrders.map((dto) => PerpOo.from(dto));
        this.accountData = undefined;
    }
    static from(publicKey, obj) {
        return new MangoAccount(publicKey, obj.group, obj.owner, obj.name, obj.delegate, obj.beingLiquidated, obj.accountNum, obj.bump, obj.netDeposits, obj.netSettled, obj.headerVersion, obj.tokens, obj.serum3, obj.perps, obj.perpOpenOrders, {});
    }
    async reload(client, group) {
        const mangoAccount = await client.getMangoAccount(this);
        await mangoAccount.reloadAccountData(client, group);
        Object.assign(this, mangoAccount);
        return mangoAccount;
    }
    async reloadWithSlot(client, group) {
        const resp = await client.getMangoAccountWithSlot(this.publicKey);
        await (resp === null || resp === void 0 ? void 0 : resp.value.reloadAccountData(client, group));
        Object.assign(this, resp === null || resp === void 0 ? void 0 : resp.value);
        return { value: resp.value, slot: resp.slot };
    }
    async reloadAccountData(client, group) {
        this.accountData = await client.computeAccountData(group, this);
        return this;
    }
    tokensActive() {
        return this.tokens.filter((token) => token.isActive());
    }
    serum3Active() {
        return this.serum3.filter((serum3) => serum3.isActive());
    }
    perpActive() {
        return this.perps.filter((perp) => perp.isActive());
    }
    perpOrdersActive() {
        return this.perpOpenOrders.filter((oo) => oo.orderMarket !== PerpOo.OrderMarketUnset);
    }
    findToken(tokenIndex) {
        return this.tokens.find((ta) => ta.tokenIndex == tokenIndex);
    }
    findSerum3Account(marketIndex) {
        return this.serum3.find((sa) => sa.marketIndex == marketIndex);
    }
    // How to navigate
    // * if a function is returning a I80F48, then usually the return value is in native quote or native token, unless specified
    // * if a function is returning a number, then usually the return value is in ui tokens, unless specified
    // * functions try to be explicit by having native or ui in the name to better reflect the value
    // * some values might appear unexpected large or small, usually the doc contains a "note"
    /**
     *
     * @param bank
     * @returns native balance for a token, is signed
     */
    getTokenBalance(bank) {
        const tp = this.findToken(bank.tokenIndex);
        return tp ? tp.balance(bank) : (0, I80F48_1.ZERO_I80F48)();
    }
    /**
     *
     * @param bank
     * @returns native deposits for a token, 0 if position has borrows
     */
    getTokenDeposits(bank) {
        const tp = this.findToken(bank.tokenIndex);
        return tp ? tp.deposits(bank) : (0, I80F48_1.ZERO_I80F48)();
    }
    /**
     *
     * @param bank
     * @returns native borrows for a token, 0 if position has deposits
     */
    getTokenBorrows(bank) {
        const tp = this.findToken(bank.tokenIndex);
        return tp ? tp.borrows(bank) : (0, I80F48_1.ZERO_I80F48)();
    }
    /**
     *
     * @param bank
     * @returns UI balance for a token, is signed
     */
    getTokenBalanceUi(bank) {
        const tp = this.findToken(bank.tokenIndex);
        return tp ? tp.balanceUi(bank) : 0;
    }
    /**
     *
     * @param bank
     * @returns UI deposits for a token, 0 or more
     */
    getTokenDepositsUi(bank) {
        const ta = this.findToken(bank.tokenIndex);
        return ta ? ta.depositsUi(bank) : 0;
    }
    /**
     *
     * @param bank
     * @returns UI borrows for a token, 0 or less
     */
    getTokenBorrowsUi(bank) {
        const ta = this.findToken(bank.tokenIndex);
        return ta ? ta.borrowsUi(bank) : 0;
    }
    /**
     * Health, see health.rs or https://docs.mango.markets/mango-markets/health-overview
     * @param healthType
     * @returns raw health number, in native quote
     */
    getHealth(healthType) {
        var _a, _b;
        return healthType == HealthType.init
            ? (_a = this.accountData) === null || _a === void 0 ? void 0 : _a.initHealth
            : (_b = this.accountData) === null || _b === void 0 ? void 0 : _b.maintHealth;
    }
    /**
     * Health ratio, which is computed so `100 * (assets-liabs)/liabs`
     * Note: health ratio is technically ∞ if liabs are 0
     * @param healthType
     * @returns health ratio, in percentage form
     */
    getHealthRatio(healthType) {
        var _a;
        return (_a = this.accountData) === null || _a === void 0 ? void 0 : _a.healthCache.healthRatio(healthType);
    }
    /**
     * Health ratio
     * @param healthType
     * @returns health ratio, in percentage form, capped to 100
     */
    getHealthRatioUi(healthType) {
        var _a;
        const ratio = (_a = this.getHealthRatio(healthType)) === null || _a === void 0 ? void 0 : _a.toNumber();
        if (ratio) {
            return ratio > 100 ? 100 : Math.trunc(ratio);
        }
        else {
            return undefined;
        }
    }
    /**
     * Sum of all the assets i.e. token deposits, borrows, total assets in spot open orders, (perps positions is todo) in terms of quote value.
     * @returns equity, in native quote
     */
    getEquity() {
        if (this.accountData) {
            const equity = this.accountData.equity;
            const total_equity = equity.tokens.reduce((a, b) => a.add(b.value), (0, I80F48_1.ZERO_I80F48)());
            return total_equity;
        }
        return undefined;
    }
    /**
     * The amount of native quote you could withdraw against your existing assets.
     * @returns collateral value, in native quote
     */
    getCollateralValue() {
        return this.getHealth(HealthType.init);
    }
    /**
     * Sum of all positive assets.
     * @returns assets, in native quote
     */
    getAssetsValue(healthType) {
        var _a;
        return (_a = this.accountData) === null || _a === void 0 ? void 0 : _a.healthCache.assets(healthType);
    }
    /**
     * Sum of all negative assets.
     * @returns liabs, in native quote
     */
    getLiabsValue(healthType) {
        var _a;
        return (_a = this.accountData) === null || _a === void 0 ? void 0 : _a.healthCache.liabs(healthType);
    }
    /**
     * The amount of given native token you can withdraw including borrows, considering all existing assets as collateral.
     * @returns amount of given native token you can borrow, considering all existing assets as collateral, in native token
     */
    getMaxWithdrawWithBorrowForToken(group, mintPk) {
        var _a;
        const tokenBank = group.getFirstBankByMint(mintPk);
        const initHealth = (_a = this.accountData) === null || _a === void 0 ? void 0 : _a.initHealth;
        if (!initHealth)
            return undefined;
        // Case 1:
        // Cannot withdraw if init health is below 0
        if (initHealth.lte((0, I80F48_1.ZERO_I80F48)())) {
            return (0, I80F48_1.ZERO_I80F48)();
        }
        // Deposits need special treatment since they would neither count towards liabilities
        // nor would be charged loanOriginationFeeRate when withdrawn
        const tp = this.findToken(tokenBank.tokenIndex);
        if (!tokenBank.price)
            return undefined;
        const existingTokenDeposits = tp ? tp.deposits(tokenBank) : (0, I80F48_1.ZERO_I80F48)();
        let existingPositionHealthContrib = (0, I80F48_1.ZERO_I80F48)();
        if (existingTokenDeposits.gt((0, I80F48_1.ZERO_I80F48)())) {
            existingPositionHealthContrib = existingTokenDeposits
                .mul(tokenBank.price)
                .imul(tokenBank.initAssetWeight);
        }
        // Case 2: token deposits have higher contribution than initHealth,
        // can withdraw without borrowing until initHealth reaches 0
        if (existingPositionHealthContrib.gt(initHealth)) {
            const withdrawAbleExistingPositionHealthContrib = initHealth;
            // console.log(`initHealth ${initHealth}`);
            // console.log(
            //   `existingPositionHealthContrib ${existingPositionHealthContrib}`,
            // );
            // console.log(
            //   `withdrawAbleExistingPositionHealthContrib ${withdrawAbleExistingPositionHealthContrib}`,
            // );
            return withdrawAbleExistingPositionHealthContrib
                .div(tokenBank.initAssetWeight)
                .div(tokenBank.price);
        }
        // Case 3: withdraw = withdraw existing deposits + borrows until initHealth reaches 0
        const initHealthWithoutExistingPosition = initHealth.sub(existingPositionHealthContrib);
        const maxBorrowNative = initHealthWithoutExistingPosition
            .div(tokenBank.initLiabWeight)
            .div(tokenBank.price);
        const maxBorrowNativeWithoutFees = maxBorrowNative.div((0, I80F48_1.ONE_I80F48)().add(tokenBank.loanOriginationFeeRate));
        // console.log(`initHealth ${initHealth}`);
        // console.log(
        //   `existingPositionHealthContrib ${existingPositionHealthContrib}`,
        // );
        // console.log(
        //   `initHealthWithoutExistingPosition ${initHealthWithoutExistingPosition}`,
        // );
        // console.log(`maxBorrowNative ${maxBorrowNative}`);
        // console.log(`maxBorrowNativeWithoutFees ${maxBorrowNativeWithoutFees}`);
        return maxBorrowNativeWithoutFees.add(existingTokenDeposits);
    }
    getMaxWithdrawWithBorrowForTokenUi(group, mintPk) {
        const maxWithdrawWithBorrow = this.getMaxWithdrawWithBorrowForToken(group, mintPk);
        if (maxWithdrawWithBorrow) {
            return (0, utils_1.toUiDecimals)(maxWithdrawWithBorrow, group.getMintDecimals(mintPk));
        }
        else {
            return undefined;
        }
    }
    /**
     * The max amount of given source native token you can swap to a target token.
     * note: slippageAndFeesFactor is a normalized number, <1,
     *  e.g. a slippage of 5% and some fees which are 1%, then slippageAndFeesFactor = 0.94
     *  the factor is used to compute how much target can be obtained by swapping source
     * @returns max amount of given source native token you can swap to a target token, in native token
     */
    getMaxSourceForTokenSwap(group, sourceMintPk, targetMintPk, slippageAndFeesFactor) {
        if (!this.accountData)
            return undefined;
        return this.accountData.healthCache
            .getMaxSourceForTokenSwap(group, sourceMintPk, targetMintPk, (0, I80F48_1.ONE_I80F48)())
            .mul(I80F48_1.I80F48.fromNumber(slippageAndFeesFactor));
    }
    /**
     * The max amount of given source ui token you can swap to a target token.
     * note: slippageAndFeesFactor is a normalized number, <1,
     *  e.g. a slippage of 5% and some fees which are 1%, then slippageAndFeesFactor = 0.94
     *  the factor is used to compute how much target can be obtained by swapping source
     * @returns max amount of given source ui token you can swap to a target token, in ui token
     */
    getMaxSourceUiForTokenSwap(group, sourceMintPk, targetMintPk, slippageAndFeesFactor) {
        const maxSource = this.getMaxSourceForTokenSwap(group, sourceMintPk, targetMintPk, slippageAndFeesFactor);
        if (maxSource) {
            return (0, utils_1.toUiDecimals)(maxSource, group.getMintDecimals(sourceMintPk));
        }
    }
    /**
     * Simulates new health ratio after applying tokenChanges to the token positions.
     * Note: token changes are expected in native amounts
     *
     * e.g. useful to simulate health after a potential swap.
     * Note: health ratio is technically ∞ if liabs are 0
     * @returns health ratio, in percentage form
     */
    simHealthRatioWithTokenPositionChanges(group, nativeTokenChanges, healthType = HealthType.init) {
        if (!this.accountData)
            return undefined;
        return this.accountData.healthCache.simHealthRatioWithTokenPositionChanges(group, nativeTokenChanges, healthType);
    }
    /**
     * Simulates new health ratio after applying tokenChanges to the token positions.
     * Note: token changes are expected in ui amounts
     *
     * e.g. useful to simulate health after a potential swap.
     * Note: health ratio is technically ∞ if liabs are 0
     * @returns health ratio, in percentage form
     */
    simHealthRatioWithTokenPositionUiChanges(group, uiTokenChanges, healthType = HealthType.init) {
        var _a;
        const nativeTokenChanges = uiTokenChanges.map((tokenChange) => {
            return {
                nativeTokenAmount: (0, utils_1.toNative)(tokenChange.uiTokenAmount, group.getMintDecimals(tokenChange.mintPk)),
                mintPk: tokenChange.mintPk,
            };
        });
        return (_a = this.accountData) === null || _a === void 0 ? void 0 : _a.healthCache.simHealthRatioWithTokenPositionChanges(group, nativeTokenChanges, healthType).toNumber();
    }
    async loadSerum3OpenOrdersAccounts(client) {
        const response = await client.program.provider.connection.getMultipleAccountsInfo(this.serum3.map((s) => s.openOrders));
        const accounts = response.filter((a) => Boolean(a));
        return accounts.map((acc, index) => {
            return market_1.OpenOrders.fromAccountInfo(this.serum3[index].openOrders, acc, constants_1.SERUM3_PROGRAM_ID[client.cluster]);
        });
    }
    async loadSerum3OpenOrdersForMarket(client, group, externalMarketPk) {
        const serum3Market = group.serum3MarketsMapByExternal.get(externalMarketPk.toBase58());
        if (!serum3Market) {
            throw new Error(`Unable to find mint serum3Market for ${externalMarketPk.toString()}`);
        }
        const serum3OO = this.serum3Active().find((s) => s.marketIndex === serum3Market.marketIndex);
        if (!serum3OO) {
            throw new Error(`No open orders account found for ${externalMarketPk}`);
        }
        const serum3MarketExternal = group.serum3MarketExternalsMap.get(externalMarketPk.toBase58());
        const [bidsInfo, asksInfo] = await client.program.provider.connection.getMultipleAccountsInfo([
            serum3MarketExternal.bidsAddress,
            serum3MarketExternal.asksAddress,
        ]);
        if (!bidsInfo || !asksInfo) {
            throw new Error(`bids and asks ai were not fetched for ${externalMarketPk.toString()}`);
        }
        const bids = market_1.Orderbook.decode(serum3MarketExternal, bidsInfo.data);
        const asks = market_1.Orderbook.decode(serum3MarketExternal, asksInfo.data);
        return [...bids, ...asks].filter((o) => o.openOrdersAddress.equals(serum3OO.openOrders));
    }
    /**
     *
     * @param group
     * @param serum3Market
     * @returns maximum native quote which can be traded for base token given current health
     */
    getMaxQuoteForSerum3Bid(group, serum3Market) {
        if (!this.accountData) {
            throw new Error(`accountData not loaded on MangoAccount, try reloading MangoAccount`);
        }
        return this.accountData.healthCache.getMaxForSerum3Order(group, serum3Market, serum3_1.Serum3Side.bid, I80F48_1.I80F48.fromNumber(1));
    }
    getMaxQuoteForSerum3BidUi(group, externalMarketPk) {
        const serum3Market = group.serum3MarketsMapByExternal.get(externalMarketPk.toBase58());
        if (!serum3Market) {
            throw new Error(`Unable to find mint serum3Market for ${externalMarketPk.toString()}`);
        }
        const nativeAmount = this.getMaxQuoteForSerum3Bid(group, serum3Market);
        return (0, utils_1.toUiDecimals)(nativeAmount, group.getFirstBankByTokenIndex(serum3Market.quoteTokenIndex).mintDecimals);
    }
    /**
     *
     * @param group
     * @param serum3Market
     * @returns maximum native base which can be traded for quote token given current health
     */
    getMaxBaseForSerum3Ask(group, serum3Market) {
        if (!this.accountData) {
            throw new Error(`accountData not loaded on MangoAccount, try reloading MangoAccount`);
        }
        return this.accountData.healthCache.getMaxForSerum3Order(group, serum3Market, serum3_1.Serum3Side.ask, I80F48_1.I80F48.fromNumber(1));
    }
    getMaxBaseForSerum3AskUi(group, externalMarketPk) {
        const serum3Market = group.serum3MarketsMapByExternal.get(externalMarketPk.toBase58());
        if (!serum3Market) {
            throw new Error(`Unable to find mint serum3Market for ${externalMarketPk.toString()}`);
        }
        const nativeAmount = this.getMaxBaseForSerum3Ask(group, serum3Market);
        return (0, utils_1.toUiDecimals)(nativeAmount, group.getFirstBankByTokenIndex(serum3Market.baseTokenIndex).mintDecimals);
    }
    /**
     *
     * @param group
     * @param nativeQuoteAmount
     * @param serum3Market
     * @param healthType
     * @returns health ratio after a bid with nativeQuoteAmount is placed
     */
    simHealthRatioWithSerum3BidChanges(group, nativeQuoteAmount, serum3Market, healthType = HealthType.init) {
        if (!this.accountData) {
            throw new Error(`accountData not loaded on MangoAccount, try reloading MangoAccount`);
        }
        return this.accountData.healthCache.simHealthRatioWithSerum3BidChanges(group, nativeQuoteAmount, serum3Market, healthType);
    }
    simHealthRatioWithSerum3BidUiChanges(group, uiQuoteAmount, externalMarketPk, healthType = HealthType.init) {
        const serum3Market = group.serum3MarketsMapByExternal.get(externalMarketPk.toBase58());
        if (!serum3Market) {
            throw new Error(`Unable to find mint serum3Market for ${externalMarketPk.toString()}`);
        }
        return this.simHealthRatioWithSerum3BidChanges(group, (0, utils_1.toNative)(uiQuoteAmount, group.getFirstBankByTokenIndex(serum3Market.quoteTokenIndex)
            .mintDecimals), serum3Market, healthType).toNumber();
    }
    /**
     *
     * @param group
     * @param nativeBaseAmount
     * @param serum3Market
     * @param healthType
     * @returns health ratio after an ask with nativeBaseAmount is placed
     */
    simHealthRatioWithSerum3AskChanges(group, nativeBaseAmount, serum3Market, healthType = HealthType.init) {
        if (!this.accountData) {
            throw new Error(`accountData not loaded on MangoAccount, try reloading MangoAccount`);
        }
        return this.accountData.healthCache.simHealthRatioWithSerum3AskChanges(group, nativeBaseAmount, serum3Market, healthType);
    }
    simHealthRatioWithSerum3AskUiChanges(group, uiBaseAmount, externalMarketPk, healthType = HealthType.init) {
        const serum3Market = group.serum3MarketsMapByExternal.get(externalMarketPk.toBase58());
        if (!serum3Market) {
            throw new Error(`Unable to find mint serum3Market for ${externalMarketPk.toString()}`);
        }
        return this.simHealthRatioWithSerum3AskChanges(group, (0, utils_1.toNative)(uiBaseAmount, group.getFirstBankByTokenIndex(serum3Market.baseTokenIndex)
            .mintDecimals), serum3Market, healthType).toNumber();
    }
    async loadPerpOpenOrdersForMarket(client, group, perpMarketName) {
        const perpMarket = group.perpMarketsMap.get(perpMarketName);
        if (!perpMarket) {
            throw new Error(`Perp Market ${perpMarketName} not found!`);
        }
        const [bids, asks] = await Promise.all([
            perpMarket.loadBids(client),
            perpMarket.loadAsks(client),
        ]);
        return [...Array.from(bids.items()), ...Array.from(asks.items())].filter((order) => order.owner.equals(this.publicKey));
    }
    toString(group) {
        let res = 'MangoAccount';
        res = res + '\n pk: ' + this.publicKey.toString();
        res = res + '\n name: ' + this.name;
        res = res + '\n owner: ' + this.owner;
        res = res + '\n delegate: ' + this.delegate;
        res =
            res +
                `\n max token slots ${this.tokens.length}, max serum3 slots ${this.serum3.length}, max perp slots ${this.perps.length}, max perp oo slots ${this.perpOpenOrders.length}`;
        res =
            this.tokensActive().length > 0
                ? res +
                    '\n tokens:' +
                    JSON.stringify(this.tokens.map((token, i) => token.isActive()
                        ? token.toString(group, i)
                        : `index: ${i} - empty slot`), null, 4)
                : res + '';
        res =
            this.serum3Active().length > 0
                ? res + '\n serum:' + JSON.stringify(this.serum3Active(), null, 4)
                : res + '';
        res =
            this.perpActive().length > 0
                ? res + '\n perps:' + JSON.stringify(this.perpActive(), null, 4)
                : res + '';
        res =
            this.perpOrdersActive().length > 0
                ? res +
                    '\n perps oo:' +
                    JSON.stringify(this.perpOrdersActive(), null, 4)
                : res + '';
        return res;
    }
}
exports.MangoAccount = MangoAccount;
class TokenPosition {
    constructor(indexedPosition, tokenIndex, inUseCount) {
        this.indexedPosition = indexedPosition;
        this.tokenIndex = tokenIndex;
        this.inUseCount = inUseCount;
    }
    static from(dto) {
        return new TokenPosition(I80F48_1.I80F48.from(dto.indexedPosition), dto.tokenIndex, dto.inUseCount);
    }
    isActive() {
        return this.tokenIndex !== TokenPosition.TokenIndexUnset;
    }
    /**
     *
     * @param bank
     * @returns native balance
     */
    balance(bank) {
        if (this.indexedPosition.isPos()) {
            return bank.depositIndex.mul(this.indexedPosition);
        }
        else {
            return bank.borrowIndex.mul(this.indexedPosition);
        }
    }
    /**
     *
     * @param bank
     * @returns native deposits, 0 if position has borrows
     */
    deposits(bank) {
        if (this.indexedPosition && this.indexedPosition.lt((0, I80F48_1.ZERO_I80F48)())) {
            return (0, I80F48_1.ZERO_I80F48)();
        }
        return this.balance(bank);
    }
    /**
     *
     * @param bank
     * @returns native borrows, 0 if position has deposits
     */
    borrows(bank) {
        if (this.indexedPosition && this.indexedPosition.gt((0, I80F48_1.ZERO_I80F48)())) {
            return (0, I80F48_1.ZERO_I80F48)();
        }
        return this.balance(bank).abs();
    }
    /**
     * @param bank
     * @returns UI balance, is signed
     */
    balanceUi(bank) {
        return (0, utils_1.nativeI80F48ToUi)(this.balance(bank), bank.mintDecimals).toNumber();
    }
    /**
     * @param bank
     * @returns UI deposits, 0 if position has borrows
     */
    depositsUi(bank) {
        return (0, utils_1.nativeI80F48ToUi)(this.deposits(bank), bank.mintDecimals).toNumber();
    }
    /**
     * @param bank
     * @returns UI borrows, 0 if position has deposits
     */
    borrowsUi(bank) {
        return (0, utils_1.nativeI80F48ToUi)(this.borrows(bank), bank.mintDecimals).toNumber();
    }
    toString(group, index) {
        let extra = '';
        if (group) {
            const bank = group.getFirstBankByTokenIndex(this.tokenIndex);
            if (bank) {
                const native = this.balance(bank);
                extra += ', native: ' + native.toNumber();
                extra += ', ui: ' + this.balanceUi(bank);
                extra += ', tokenName: ' + bank.name;
            }
        }
        return ((index !== undefined ? 'index: ' + index : '') +
            ', tokenIndex: ' +
            this.tokenIndex +
            ', inUseCount: ' +
            this.inUseCount +
            ', indexedValue: ' +
            this.indexedPosition.toNumber() +
            extra);
    }
}
exports.TokenPosition = TokenPosition;
TokenPosition.TokenIndexUnset = 65535;
class TokenPositionDto {
    constructor(indexedPosition, tokenIndex, inUseCount, reserved) {
        this.indexedPosition = indexedPosition;
        this.tokenIndex = tokenIndex;
        this.inUseCount = inUseCount;
        this.reserved = reserved;
    }
}
exports.TokenPositionDto = TokenPositionDto;
class Serum3Orders {
    constructor(openOrders, marketIndex, baseTokenIndex, quoteTokenIndex) {
        this.openOrders = openOrders;
        this.marketIndex = marketIndex;
        this.baseTokenIndex = baseTokenIndex;
        this.quoteTokenIndex = quoteTokenIndex;
    }
    static from(dto) {
        return new Serum3Orders(dto.openOrders, dto.marketIndex, dto.baseTokenIndex, dto.quoteTokenIndex);
    }
    isActive() {
        return this.marketIndex !== Serum3Orders.Serum3MarketIndexUnset;
    }
}
exports.Serum3Orders = Serum3Orders;
Serum3Orders.Serum3MarketIndexUnset = 65535;
class Serum3PositionDto {
    constructor(openOrders, marketIndex, baseTokenIndex, quoteTokenIndex, reserved) {
        this.openOrders = openOrders;
        this.marketIndex = marketIndex;
        this.baseTokenIndex = baseTokenIndex;
        this.quoteTokenIndex = quoteTokenIndex;
        this.reserved = reserved;
    }
}
exports.Serum3PositionDto = Serum3PositionDto;
class PerpPosition {
    constructor(marketIndex, basePositionLots, quotePositionNative, bidsBaseLots, asksBaseLots, takerBaseLots, takerQuoteLots) {
        this.marketIndex = marketIndex;
        this.basePositionLots = basePositionLots;
        this.quotePositionNative = quotePositionNative;
        this.bidsBaseLots = bidsBaseLots;
        this.asksBaseLots = asksBaseLots;
        this.takerBaseLots = takerBaseLots;
        this.takerQuoteLots = takerQuoteLots;
    }
    static from(dto) {
        return new PerpPosition(dto.marketIndex, dto.basePositionLots.toNumber(), dto.quotePositionNative.val, dto.bidsBaseLots.toNumber(), dto.asksBaseLots.toNumber(), dto.takerBaseLots.toNumber(), dto.takerQuoteLots.toNumber());
    }
    isActive() {
        return this.marketIndex != PerpPosition.PerpMarketIndexUnset;
    }
}
exports.PerpPosition = PerpPosition;
PerpPosition.PerpMarketIndexUnset = 65535;
class PerpPositionDto {
    constructor(marketIndex, reserved, basePositionLots, quotePositionNative, bidsBaseLots, asksBaseLots, takerBaseLots, takerQuoteLots) {
        this.marketIndex = marketIndex;
        this.reserved = reserved;
        this.basePositionLots = basePositionLots;
        this.quotePositionNative = quotePositionNative;
        this.bidsBaseLots = bidsBaseLots;
        this.asksBaseLots = asksBaseLots;
        this.takerBaseLots = takerBaseLots;
        this.takerQuoteLots = takerQuoteLots;
    }
}
exports.PerpPositionDto = PerpPositionDto;
class PerpOo {
    constructor(orderSide, orderMarket, clientOrderId, orderId) {
        this.orderSide = orderSide;
        this.orderMarket = orderMarket;
        this.clientOrderId = clientOrderId;
        this.orderId = orderId;
    }
    static from(dto) {
        return new PerpOo(dto.orderSide, dto.orderMarket, dto.clientOrderId.toNumber(), dto.orderId);
    }
}
exports.PerpOo = PerpOo;
PerpOo.OrderMarketUnset = 65535;
class PerpOoDto {
    constructor(orderSide, orderMarket, clientOrderId, orderId) {
        this.orderSide = orderSide;
        this.orderMarket = orderMarket;
        this.clientOrderId = clientOrderId;
        this.orderId = orderId;
    }
}
exports.PerpOoDto = PerpOoDto;
class HealthType {
}
exports.HealthType = HealthType;
HealthType.maint = { maint: {} };
HealthType.init = { init: {} };
class MangoAccountData {
    constructor(healthCache, initHealth, maintHealth, equity) {
        this.healthCache = healthCache;
        this.initHealth = initHealth;
        this.maintHealth = maintHealth;
        this.equity = equity;
    }
    static from(event) {
        return new MangoAccountData(healthCache_1.HealthCache.fromDto(event.healthCache), I80F48_1.I80F48.from(event.initHealth), I80F48_1.I80F48.from(event.maintHealth), Equity.from(event.equity));
    }
}
exports.MangoAccountData = MangoAccountData;
class Equity {
    constructor(tokens, perps) {
        this.tokens = tokens;
        this.perps = perps;
    }
    static from(dto) {
        return new Equity(dto.tokens.map((token) => new TokenEquity(token.tokenIndex, I80F48_1.I80F48.from(token.value))), dto.perps.map((perpAccount) => new PerpEquity(perpAccount.perpMarketIndex, I80F48_1.I80F48.from(perpAccount.value))));
    }
}
exports.Equity = Equity;
class TokenEquity {
    constructor(tokenIndex, value) {
        this.tokenIndex = tokenIndex;
        this.value = value;
    }
}
exports.TokenEquity = TokenEquity;
class PerpEquity {
    constructor(perpMarketIndex, value) {
        this.perpMarketIndex = perpMarketIndex;
        this.value = value;
    }
}
exports.PerpEquity = PerpEquity;
class EquityDto {
}
exports.EquityDto = EquityDto;
//# sourceMappingURL=mangoAccount.js.map