"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSerum3MarketExternalVaultSignerAddress = exports.Serum3Side = exports.Serum3OrderType = exports.Serum3SelfTradeBehavior = exports.Serum3Market = void 0;
const bytes_1 = require("@project-serum/anchor/dist/cjs/utils/bytes");
const web3_js_1 = require("@solana/web3.js");
const constants_1 = require("../constants");
const I80F48_1 = require("./I80F48");
class Serum3Market {
    constructor(publicKey, group, baseTokenIndex, quoteTokenIndex, name, serumProgram, serumMarketExternal, marketIndex, registrationTime) {
        this.publicKey = publicKey;
        this.group = group;
        this.baseTokenIndex = baseTokenIndex;
        this.quoteTokenIndex = quoteTokenIndex;
        this.serumProgram = serumProgram;
        this.serumMarketExternal = serumMarketExternal;
        this.marketIndex = marketIndex;
        this.registrationTime = registrationTime;
        this.name = bytes_1.utf8.decode(new Uint8Array(name)).split('\x00')[0];
    }
    static from(publicKey, obj) {
        return new Serum3Market(publicKey, obj.group, obj.baseTokenIndex, obj.quoteTokenIndex, obj.name, obj.serumProgram, obj.serumMarketExternal, obj.marketIndex, obj.registrationTime);
    }
    /**
     *
     * @param group
     * @returns maximum leverage one can bid on this market, this is only for display purposes,
     *  also see getMaxQuoteForSerum3BidUi and getMaxBaseForSerum3AskUi
     */
    maxBidLeverage(group) {
        const baseBank = group.getFirstBankByTokenIndex(this.baseTokenIndex);
        if (!baseBank) {
            throw new Error(`bank for base token with index ${this.baseTokenIndex} not found`);
        }
        const quoteBank = group.getFirstBankByTokenIndex(this.quoteTokenIndex);
        if (!quoteBank) {
            throw new Error(`bank for quote token with index ${this.quoteTokenIndex} not found`);
        }
        if (quoteBank.initLiabWeight.sub(baseBank.initAssetWeight).lte((0, I80F48_1.ZERO_I80F48)())) {
            return (0, I80F48_1.MAX_I80F48)().toNumber();
        }
        return (0, I80F48_1.ONE_I80F48)()
            .div(quoteBank.initLiabWeight.sub(baseBank.initAssetWeight))
            .toNumber();
    }
    /**
     *
     * @param group
     * @returns maximum leverage one can ask on this market, this is only for display purposes,
     *  also see getMaxQuoteForSerum3BidUi and getMaxBaseForSerum3AskUi
     */
    maxAskLeverage(group) {
        const baseBank = group.getFirstBankByTokenIndex(this.baseTokenIndex);
        if (!baseBank) {
            throw new Error(`bank for base token with index ${this.baseTokenIndex} not found`);
        }
        const quoteBank = group.getFirstBankByTokenIndex(this.quoteTokenIndex);
        if (!quoteBank) {
            throw new Error(`bank for quote token with index ${this.quoteTokenIndex} not found`);
        }
        if (baseBank.initLiabWeight.sub(quoteBank.initAssetWeight).lte((0, I80F48_1.ZERO_I80F48)())) {
            return (0, I80F48_1.MAX_I80F48)().toNumber();
        }
        return (0, I80F48_1.ONE_I80F48)()
            .div(baseBank.initLiabWeight.sub(quoteBank.initAssetWeight))
            .toNumber();
    }
    getSerum3ExternalMarket(group) {
        return group.serum3MarketExternalsMap.get(this.serumMarketExternal.toBase58());
    }
    async loadBids(client, group) {
        const serum3MarketExternal = group.serum3MarketExternalsMap.get(this.serumMarketExternal.toBase58());
        if (!serum3MarketExternal) {
            throw new Error(`Unable to find serum3MarketExternal for ${this.serumMarketExternal.toBase58()}`);
        }
        return await serum3MarketExternal.loadBids(client.program.provider.connection);
    }
    async loadAsks(client, group) {
        const serum3MarketExternal = group.serum3MarketExternalsMap.get(this.serumMarketExternal.toBase58());
        if (!serum3MarketExternal) {
            throw new Error(`Unable to find serum3MarketExternal for ${this.serumMarketExternal.toBase58()}`);
        }
        return await serum3MarketExternal.loadAsks(client.program.provider.connection);
    }
    async logOb(client, group) {
        let res = ``;
        res += `  ${this.name} OrderBook`;
        let orders = await (this === null || this === void 0 ? void 0 : this.loadAsks(client, group));
        for (const order of orders.items(true)) {
            res += `\n  ${order.price.toString().padStart(10)}, ${order.size
                .toString()
                .padStart(10)}`;
        }
        res += `\n  --------------------------`;
        orders = await (this === null || this === void 0 ? void 0 : this.loadBids(client, group));
        for (const order of orders.items(true)) {
            res += `\n  ${order.price.toString().padStart(10)}, ${order.size
                .toString()
                .padStart(10)}`;
        }
        return res;
    }
}
exports.Serum3Market = Serum3Market;
class Serum3SelfTradeBehavior {
}
exports.Serum3SelfTradeBehavior = Serum3SelfTradeBehavior;
Serum3SelfTradeBehavior.decrementTake = { decrementTake: {} };
Serum3SelfTradeBehavior.cancelProvide = { cancelProvide: {} };
Serum3SelfTradeBehavior.abortTransaction = { abortTransaction: {} };
class Serum3OrderType {
}
exports.Serum3OrderType = Serum3OrderType;
Serum3OrderType.limit = { limit: {} };
Serum3OrderType.immediateOrCancel = { immediateOrCancel: {} };
Serum3OrderType.postOnly = { postOnly: {} };
class Serum3Side {
}
exports.Serum3Side = Serum3Side;
Serum3Side.bid = { bid: {} };
Serum3Side.ask = { ask: {} };
async function generateSerum3MarketExternalVaultSignerAddress(cluster, serum3Market, serum3MarketExternal) {
    return await web3_js_1.PublicKey.createProgramAddress([
        serum3Market.serumMarketExternal.toBuffer(),
        serum3MarketExternal.decoded.vaultSignerNonce.toArrayLike(Buffer, 'le', 8),
    ], constants_1.SERUM3_PROGRAM_ID[cluster]);
}
exports.generateSerum3MarketExternalVaultSignerAddress = generateSerum3MarketExternalVaultSignerAddress;
//# sourceMappingURL=serum3.js.map