import { BN } from '@project-serum/anchor';
import { utf8 } from '@project-serum/anchor/dist/cjs/utils/bytes';
import Big from 'big.js';
import { U64_MAX_BN } from '../utils';
import { QUOTE_DECIMALS } from './bank';
import { I80F48 } from './I80F48';
export class PerpMarket {
    constructor(publicKey, group, quoteTokenIndex, perpMarketIndex, name, oracle, oracleConfig, bids, asks, eventQueue, quoteLotSize, baseLotSize, maintAssetWeight, initAssetWeight, maintLiabWeight, initLiabWeight, liquidationFee, makerFee, takerFee, minFunding, maxFunding, impactQuantity, longFunding, shortFunding, fundingLastUpdated, openInterest, seqNum, feesAccrued, bump, baseDecimals, registrationTime) {
        this.publicKey = publicKey;
        this.group = group;
        this.quoteTokenIndex = quoteTokenIndex;
        this.perpMarketIndex = perpMarketIndex;
        this.oracle = oracle;
        this.bids = bids;
        this.asks = asks;
        this.eventQueue = eventQueue;
        this.quoteLotSize = quoteLotSize;
        this.baseLotSize = baseLotSize;
        this.impactQuantity = impactQuantity;
        this.baseDecimals = baseDecimals;
        this.registrationTime = registrationTime;
        this.name = utf8.decode(new Uint8Array(name)).split('\x00')[0];
        this.maintAssetWeight = I80F48.from(maintAssetWeight);
        this.initAssetWeight = I80F48.from(initAssetWeight);
        this.maintLiabWeight = I80F48.from(maintLiabWeight);
        this.initLiabWeight = I80F48.from(initLiabWeight);
        this.liquidationFee = I80F48.from(liquidationFee);
        this.makerFee = I80F48.from(makerFee);
        this.takerFee = I80F48.from(takerFee);
        this.minFunding = I80F48.from(minFunding);
        this.maxFunding = I80F48.from(maxFunding);
        this.openInterest = openInterest.toNumber();
        this.seqNum = seqNum.toNumber();
        this.feesAccrued = I80F48.from(feesAccrued);
        this.priceLotsToUiConverter = new Big(10)
            .pow(baseDecimals - QUOTE_DECIMALS)
            .mul(new Big(this.quoteLotSize.toString()))
            .div(new Big(this.baseLotSize.toString()))
            .toNumber();
        this.baseLotsToUiConverter = new Big(this.baseLotSize.toString())
            .div(new Big(10).pow(baseDecimals))
            .toNumber();
        this.quoteLotsToUiConverter = new Big(this.quoteLotSize.toString())
            .div(new Big(10).pow(QUOTE_DECIMALS))
            .toNumber();
    }
    static from(publicKey, obj) {
        return new PerpMarket(publicKey, obj.group, obj.quoteTokenIndex, obj.perpMarketIndex, obj.name, obj.oracle, obj.oracleConfig, obj.bids, obj.asks, obj.eventQueue, obj.quoteLotSize, obj.baseLotSize, obj.maintAssetWeight, obj.initAssetWeight, obj.maintLiabWeight, obj.initLiabWeight, obj.liquidationFee, obj.makerFee, obj.takerFee, obj.minFunding, obj.maxFunding, obj.impactQuantity, obj.longFunding, obj.shortFunding, obj.fundingLastUpdated, obj.openInterest, obj.seqNum, obj.feesAccrued, obj.bump, obj.baseDecimals, obj.registrationTime);
    }
    async loadAsks(client) {
        const asks = await client.program.account.bookSide.fetch(this.asks);
        return BookSide.from(client, this, BookSideType.asks, asks);
    }
    async loadBids(client) {
        const bids = await client.program.account.bookSide.fetch(this.bids);
        return BookSide.from(client, this, BookSideType.bids, bids);
    }
    async loadEventQueue(client) {
        const eventQueue = await client.program.account.eventQueue.fetch(this.eventQueue);
        return new PerpEventQueue(client, eventQueue.header, eventQueue.buf);
    }
    async loadFills(client, lastSeqNum) {
        const eventQueue = await this.loadEventQueue(client);
        return eventQueue
            .eventsSince(lastSeqNum)
            .filter((event) => event.eventType == PerpEventQueue.FILL_EVENT_TYPE);
    }
    /**
     *
     * @param bids
     * @param asks
     * @returns returns funding rate per hour
     */
    getCurrentFundingRate(bids, asks) {
        const MIN_FUNDING = this.minFunding.toNumber();
        const MAX_FUNDING = this.maxFunding.toNumber();
        const bid = bids.getImpactPriceUi(new BN(this.impactQuantity));
        const ask = asks.getImpactPriceUi(new BN(this.impactQuantity));
        const indexPrice = this.uiPrice;
        let funding;
        if (bid !== undefined && ask !== undefined) {
            const bookPrice = (bid + ask) / 2;
            funding = Math.min(Math.max(bookPrice / indexPrice - 1, MIN_FUNDING), MAX_FUNDING);
        }
        else if (bid !== undefined) {
            funding = MAX_FUNDING;
        }
        else if (ask !== undefined) {
            funding = MIN_FUNDING;
        }
        else {
            funding = 0;
        }
        return funding / 24;
    }
    uiPriceToLots(price) {
        return new BN(price * Math.pow(10, QUOTE_DECIMALS))
            .mul(this.baseLotSize)
            .div(this.quoteLotSize.mul(new BN(Math.pow(10, this.baseDecimals))));
    }
    uiBaseToLots(quantity) {
        return new BN(quantity * Math.pow(10, this.baseDecimals)).div(this.baseLotSize);
    }
    uiQuoteToLots(uiQuote) {
        return new BN(uiQuote * Math.pow(10, QUOTE_DECIMALS)).div(this.quoteLotSize);
    }
    priceLotsToUi(price) {
        return parseFloat(price.toString()) * this.priceLotsToUiConverter;
    }
    baseLotsToUi(quantity) {
        return parseFloat(quantity.toString()) * this.baseLotsToUiConverter;
    }
    quoteLotsToUi(quantity) {
        return parseFloat(quantity.toString()) * this.quoteLotsToUiConverter;
    }
    toString() {
        return ('PerpMarket ' +
            '\n perpMarketIndex -' +
            this.perpMarketIndex +
            '\n maintAssetWeight -' +
            this.maintAssetWeight.toNumber() +
            '\n initAssetWeight -' +
            this.initAssetWeight.toNumber() +
            '\n maintLiabWeight -' +
            this.maintLiabWeight.toNumber() +
            '\n initLiabWeight -' +
            this.initLiabWeight.toNumber() +
            '\n liquidationFee -' +
            this.liquidationFee.toNumber() +
            '\n makerFee -' +
            this.makerFee.toNumber() +
            '\n takerFee -' +
            this.takerFee.toNumber());
    }
}
export class BookSide {
    constructor(client, perpMarket, type, bumpIndex, freeListLen, freeListHead, rootNode, leafCount, nodes, includeExpired = false, maxBookDelay) {
        this.client = client;
        this.perpMarket = perpMarket;
        this.type = type;
        this.bumpIndex = bumpIndex;
        this.freeListLen = freeListLen;
        this.freeListHead = freeListHead;
        this.rootNode = rootNode;
        this.leafCount = leafCount;
        this.nodes = nodes;
        this.includeExpired = includeExpired;
        // TODO why? Ask Daffy
        // Determine the maxTimestamp found on the book to use for tif
        // If maxBookDelay is not provided, use 3600 as a very large number
        maxBookDelay = maxBookDelay === undefined ? 3600 : maxBookDelay;
        let maxTimestamp = new BN(new Date().getTime() / 1000 - maxBookDelay);
        for (const node of this.nodes) {
            if (node.tag !== BookSide.LEAF_NODE_TAG) {
                continue;
            }
            const leafNode = BookSide.toLeafNode(client, node.data);
            if (leafNode.timestamp.gt(maxTimestamp)) {
                maxTimestamp = leafNode.timestamp;
            }
        }
        this.now = maxTimestamp;
    }
    static from(client, perpMarket, bookSideType, obj) {
        return new BookSide(client, perpMarket, bookSideType, obj.bumpIndex, obj.freeListLen, obj.freeListHead, obj.rootNode, obj.leafCount, obj.nodes);
    }
    static getPriceFromKey(key) {
        return key.ushrn(64);
    }
    *items() {
        if (this.leafCount === 0) {
            return;
        }
        const now = this.now;
        const stack = [this.rootNode];
        const [left, right] = this.type === BookSideType.bids ? [1, 0] : [0, 1];
        while (stack.length > 0) {
            const index = stack.pop();
            const node = this.nodes[index];
            if (node.tag === BookSide.INNER_NODE_TAG) {
                const innerNode = BookSide.toInnerNode(this.client, node.data);
                stack.push(innerNode.children[right], innerNode.children[left]);
            }
            else if (node.tag === BookSide.LEAF_NODE_TAG) {
                const leafNode = BookSide.toLeafNode(this.client, node.data);
                const expiryTimestamp = leafNode.timeInForce
                    ? leafNode.timestamp.add(new BN(leafNode.timeInForce))
                    : U64_MAX_BN;
                if (now.lt(expiryTimestamp) || this.includeExpired) {
                    yield PerpOrder.from(this.perpMarket, leafNode, this.type);
                }
            }
        }
    }
    getImpactPriceUi(baseLots) {
        const s = new BN(0);
        for (const order of this.items()) {
            s.iadd(order.sizeLots);
            if (s.gte(baseLots)) {
                return order.price;
            }
        }
        return undefined;
    }
    getL2(depth) {
        const levels = [];
        for (const { priceLots, sizeLots } of this.items()) {
            if (levels.length > 0 && levels[levels.length - 1][0].eq(priceLots)) {
                levels[levels.length - 1][1].iadd(sizeLots);
            }
            else if (levels.length === depth) {
                break;
            }
            else {
                levels.push([priceLots, sizeLots]);
            }
        }
        return levels.map(([priceLots, sizeLots]) => [
            this.perpMarket.priceLotsToUi(priceLots),
            this.perpMarket.baseLotsToUi(sizeLots),
            priceLots,
            sizeLots,
        ]);
    }
    getL2Ui(depth) {
        const levels = [];
        for (const { price, size } of this.items()) {
            if (levels.length > 0 && levels[levels.length - 1][0] === price) {
                levels[levels.length - 1][1] += size;
            }
            else if (levels.length === depth) {
                break;
            }
            else {
                levels.push([price, size]);
            }
        }
        return levels;
    }
    static toInnerNode(client, data) {
        return client.program._coder.types.typeLayouts
            .get('InnerNode')
            .decode(Buffer.from([BookSide.INNER_NODE_TAG, 0, 0, 0].concat(data)));
    }
    static toLeafNode(client, data) {
        return LeafNode.from(client.program._coder.types.typeLayouts
            .get('LeafNode')
            .decode(Buffer.from([BookSide.LEAF_NODE_TAG, 0, 0, 0].concat(data))));
    }
}
BookSide.INNER_NODE_TAG = 1;
BookSide.LEAF_NODE_TAG = 2;
export class BookSideType {
}
BookSideType.bids = { bids: {} };
BookSideType.asks = { asks: {} };
export class LeafNode {
    constructor(ownerSlot, orderType, timeInForce, key, owner, quantity, clientOrderId, timestamp) {
        this.ownerSlot = ownerSlot;
        this.orderType = orderType;
        this.timeInForce = timeInForce;
        this.key = key;
        this.owner = owner;
        this.quantity = quantity;
        this.clientOrderId = clientOrderId;
        this.timestamp = timestamp;
    }
    static from(obj) {
        return new LeafNode(obj.ownerSlot, obj.orderType, obj.timeInForce, obj.key, obj.owner, obj.quantity, obj.clientOrderId, obj.timestamp);
    }
}
export class InnerNode {
    constructor(children) {
        this.children = children;
    }
    static from(obj) {
        return new InnerNode(obj.children);
    }
}
export class Side {
}
Side.bid = { bid: {} };
Side.ask = { ask: {} };
export class PerpOrderType {
}
PerpOrderType.limit = { limit: {} };
PerpOrderType.immediateOrCancel = { immediateorcancel: {} };
PerpOrderType.postOnly = { postonly: {} };
PerpOrderType.market = { market: {} };
PerpOrderType.postOnlySlide = { postonlyslide: {} };
export class PerpOrder {
    constructor(orderId, clientId, owner, openOrdersSlot, feeTier, price, priceLots, size, sizeLots, side, timestamp, expiryTimestamp) {
        this.orderId = orderId;
        this.clientId = clientId;
        this.owner = owner;
        this.openOrdersSlot = openOrdersSlot;
        this.feeTier = feeTier;
        this.price = price;
        this.priceLots = priceLots;
        this.size = size;
        this.sizeLots = sizeLots;
        this.side = side;
        this.timestamp = timestamp;
        this.expiryTimestamp = expiryTimestamp;
    }
    static from(perpMarket, leafNode, type) {
        const side = type == BookSideType.bids ? Side.bid : Side.ask;
        const price = BookSide.getPriceFromKey(leafNode.key);
        const expiryTimestamp = leafNode.timeInForce
            ? leafNode.timestamp.add(new BN(leafNode.timeInForce))
            : U64_MAX_BN;
        return new PerpOrder(leafNode.key, leafNode.clientOrderId, leafNode.owner, leafNode.ownerSlot, 0, perpMarket.priceLotsToUi(price), price, perpMarket.baseLotsToUi(leafNode.quantity), leafNode.quantity, side, leafNode.timestamp, expiryTimestamp);
    }
}
export class PerpEventQueue {
    constructor(client, header, buf) {
        this.head = header.head;
        this.count = header.count;
        this.seqNum = header.seqNum;
        this.rawEvents = buf.slice(this.head, this.count).map((event) => {
            if (event.eventType === PerpEventQueue.FILL_EVENT_TYPE) {
                return client.program._coder.types.typeLayouts
                    .get('FillEvent')
                    .decode(Buffer.from([PerpEventQueue.FILL_EVENT_TYPE].concat(event.padding)));
            }
            else if (event.eventType === PerpEventQueue.OUT_EVENT_TYPE) {
                return client.program._coder.types.typeLayouts
                    .get('OutEvent')
                    .decode(Buffer.from([PerpEventQueue.OUT_EVENT_TYPE].concat(event.padding)));
            }
            else if (event.eventType === PerpEventQueue.LIQUIDATE_EVENT_TYPE) {
                return client.program._coder.types.typeLayouts
                    .get('LiquidateEvent')
                    .decode(Buffer.from([PerpEventQueue.LIQUIDATE_EVENT_TYPE].concat(event.padding)));
            }
            throw new Error(`Unknown event with eventType ${event.eventType}`);
        });
    }
    getUnconsumedEvents() {
        const events = [];
        const head = this.head;
        for (let i = 0; i < this.count; i++) {
            events.push(this.rawEvents[(head + i) % this.rawEvents.length]);
        }
        return events;
    }
    eventsSince(lastSeqNum) {
        return this.rawEvents
            .filter((e) => e.seqNum.gt(lastSeqNum === undefined ? new BN(0) : lastSeqNum))
            .sort((a, b) => a.seqNum.cmp(b.seqNum));
    }
}
PerpEventQueue.FILL_EVENT_TYPE = 0;
PerpEventQueue.OUT_EVENT_TYPE = 1;
PerpEventQueue.LIQUIDATE_EVENT_TYPE = 2;
//# sourceMappingURL=perp.js.map