import { utf8 } from '@project-serum/anchor/dist/cjs/utils/bytes';
import { PublicKey } from '@solana/web3.js';
import { SERUM3_PROGRAM_ID } from '../constants';
import { MAX_I80F48, ONE_I80F48, ZERO_I80F48 } from './I80F48';
export class Serum3Market {
    constructor(publicKey, group, baseTokenIndex, quoteTokenIndex, name, serumProgram, serumMarketExternal, marketIndex, registrationTime) {
        this.publicKey = publicKey;
        this.group = group;
        this.baseTokenIndex = baseTokenIndex;
        this.quoteTokenIndex = quoteTokenIndex;
        this.serumProgram = serumProgram;
        this.serumMarketExternal = serumMarketExternal;
        this.marketIndex = marketIndex;
        this.registrationTime = registrationTime;
        this.name = utf8.decode(new Uint8Array(name)).split('\x00')[0];
    }
    static from(publicKey, obj) {
        return new Serum3Market(publicKey, obj.group, obj.baseTokenIndex, obj.quoteTokenIndex, obj.name, obj.serumProgram, obj.serumMarketExternal, obj.marketIndex, obj.registrationTime);
    }
    /**
     *
     * @param group
     * @returns maximum leverage one can bid on this market, this is only for display purposes,
     *  also see getMaxQuoteForSerum3BidUi and getMaxBaseForSerum3AskUi
     */
    maxBidLeverage(group) {
        const baseBank = group.getFirstBankByTokenIndex(this.baseTokenIndex);
        if (!baseBank) {
            throw new Error(`bank for base token with index ${this.baseTokenIndex} not found`);
        }
        const quoteBank = group.getFirstBankByTokenIndex(this.quoteTokenIndex);
        if (!quoteBank) {
            throw new Error(`bank for quote token with index ${this.quoteTokenIndex} not found`);
        }
        if (quoteBank.initLiabWeight.sub(baseBank.initAssetWeight).lte(ZERO_I80F48())) {
            return MAX_I80F48().toNumber();
        }
        return ONE_I80F48()
            .div(quoteBank.initLiabWeight.sub(baseBank.initAssetWeight))
            .toNumber();
    }
    /**
     *
     * @param group
     * @returns maximum leverage one can ask on this market, this is only for display purposes,
     *  also see getMaxQuoteForSerum3BidUi and getMaxBaseForSerum3AskUi
     */
    maxAskLeverage(group) {
        const baseBank = group.getFirstBankByTokenIndex(this.baseTokenIndex);
        if (!baseBank) {
            throw new Error(`bank for base token with index ${this.baseTokenIndex} not found`);
        }
        const quoteBank = group.getFirstBankByTokenIndex(this.quoteTokenIndex);
        if (!quoteBank) {
            throw new Error(`bank for quote token with index ${this.quoteTokenIndex} not found`);
        }
        if (baseBank.initLiabWeight.sub(quoteBank.initAssetWeight).lte(ZERO_I80F48())) {
            return MAX_I80F48().toNumber();
        }
        return ONE_I80F48()
            .div(baseBank.initLiabWeight.sub(quoteBank.initAssetWeight))
            .toNumber();
    }
    getSerum3ExternalMarket(group) {
        return group.serum3MarketExternalsMap.get(this.serumMarketExternal.toBase58());
    }
    async loadBids(client, group) {
        const serum3MarketExternal = group.serum3MarketExternalsMap.get(this.serumMarketExternal.toBase58());
        if (!serum3MarketExternal) {
            throw new Error(`Unable to find serum3MarketExternal for ${this.serumMarketExternal.toBase58()}`);
        }
        return await serum3MarketExternal.loadBids(client.program.provider.connection);
    }
    async loadAsks(client, group) {
        const serum3MarketExternal = group.serum3MarketExternalsMap.get(this.serumMarketExternal.toBase58());
        if (!serum3MarketExternal) {
            throw new Error(`Unable to find serum3MarketExternal for ${this.serumMarketExternal.toBase58()}`);
        }
        return await serum3MarketExternal.loadAsks(client.program.provider.connection);
    }
    async logOb(client, group) {
        let res = ``;
        res += `  ${this.name} OrderBook`;
        let orders = await (this === null || this === void 0 ? void 0 : this.loadAsks(client, group));
        for (const order of orders.items(true)) {
            res += `\n  ${order.price.toString().padStart(10)}, ${order.size
                .toString()
                .padStart(10)}`;
        }
        res += `\n  --------------------------`;
        orders = await (this === null || this === void 0 ? void 0 : this.loadBids(client, group));
        for (const order of orders.items(true)) {
            res += `\n  ${order.price.toString().padStart(10)}, ${order.size
                .toString()
                .padStart(10)}`;
        }
        return res;
    }
}
export class Serum3SelfTradeBehavior {
}
Serum3SelfTradeBehavior.decrementTake = { decrementTake: {} };
Serum3SelfTradeBehavior.cancelProvide = { cancelProvide: {} };
Serum3SelfTradeBehavior.abortTransaction = { abortTransaction: {} };
export class Serum3OrderType {
}
Serum3OrderType.limit = { limit: {} };
Serum3OrderType.immediateOrCancel = { immediateOrCancel: {} };
Serum3OrderType.postOnly = { postOnly: {} };
export class Serum3Side {
}
Serum3Side.bid = { bid: {} };
Serum3Side.ask = { ask: {} };
export async function generateSerum3MarketExternalVaultSignerAddress(cluster, serum3Market, serum3MarketExternal) {
    return await PublicKey.createProgramAddress([
        serum3Market.serumMarketExternal.toBuffer(),
        serum3MarketExternal.decoded.vaultSignerNonce.toArrayLike(Buffer, 'le', 8),
    ], SERUM3_PROGRAM_ID[cluster]);
}
//# sourceMappingURL=serum3.js.map