/// <reference types="bn.js" />
import { BN } from '@project-serum/anchor';
import { OpenOrders, Order } from '@project-serum/serum/lib/market';
import { PublicKey } from '@solana/web3.js';
import { MangoClient } from '../client';
import { Bank } from './bank';
import { Group } from './group';
import { HealthCache, HealthCacheDto } from './healthCache';
import { I80F48, I80F48Dto } from './I80F48';
import { PerpOrder } from './perp';
import { Serum3Market } from './serum3';
export declare class MangoAccount {
    publicKey: PublicKey;
    group: PublicKey;
    owner: PublicKey;
    delegate: PublicKey;
    accountNum: number;
    accountData: undefined | MangoAccountData;
    tokens: TokenPosition[];
    serum3: Serum3Orders[];
    perps: PerpPosition[];
    perpOpenOrders: PerpOo[];
    name: string;
    static from(publicKey: PublicKey, obj: {
        group: PublicKey;
        owner: PublicKey;
        name: number[];
        delegate: PublicKey;
        beingLiquidated: number;
        accountNum: number;
        bump: number;
        netDeposits: BN;
        netSettled: BN;
        headerVersion: number;
        tokens: unknown;
        serum3: unknown;
        perps: unknown;
        perpOpenOrders: unknown;
    }): MangoAccount;
    constructor(publicKey: PublicKey, group: PublicKey, owner: PublicKey, name: number[], delegate: PublicKey, beingLiquidated: number, accountNum: number, bump: number, netDeposits: BN, netSettled: BN, headerVersion: number, tokens: TokenPositionDto[], serum3: Serum3PositionDto[], perps: PerpPositionDto[], perpOpenOrders: PerpOoDto[], accountData: undefined | MangoAccountData);
    reload(client: MangoClient, group: Group): Promise<MangoAccount>;
    reloadWithSlot(client: MangoClient, group: Group): Promise<{
        value: MangoAccount;
        slot: number;
    }>;
    reloadAccountData(client: MangoClient, group: Group): Promise<MangoAccount>;
    tokensActive(): TokenPosition[];
    serum3Active(): Serum3Orders[];
    perpActive(): PerpPosition[];
    perpOrdersActive(): PerpOo[];
    findToken(tokenIndex: number): TokenPosition | undefined;
    findSerum3Account(marketIndex: number): Serum3Orders | undefined;
    /**
     *
     * @param bank
     * @returns native balance for a token, is signed
     */
    getTokenBalance(bank: Bank): I80F48;
    /**
     *
     * @param bank
     * @returns native deposits for a token, 0 if position has borrows
     */
    getTokenDeposits(bank: Bank): I80F48;
    /**
     *
     * @param bank
     * @returns native borrows for a token, 0 if position has deposits
     */
    getTokenBorrows(bank: Bank): I80F48;
    /**
     *
     * @param bank
     * @returns UI balance for a token, is signed
     */
    getTokenBalanceUi(bank: Bank): number;
    /**
     *
     * @param bank
     * @returns UI deposits for a token, 0 or more
     */
    getTokenDepositsUi(bank: Bank): number;
    /**
     *
     * @param bank
     * @returns UI borrows for a token, 0 or less
     */
    getTokenBorrowsUi(bank: Bank): number;
    /**
     * Health, see health.rs or https://docs.mango.markets/mango-markets/health-overview
     * @param healthType
     * @returns raw health number, in native quote
     */
    getHealth(healthType: HealthType): I80F48 | undefined;
    /**
     * Health ratio, which is computed so `100 * (assets-liabs)/liabs`
     * Note: health ratio is technically ∞ if liabs are 0
     * @param healthType
     * @returns health ratio, in percentage form
     */
    getHealthRatio(healthType: HealthType): I80F48 | undefined;
    /**
     * Health ratio
     * @param healthType
     * @returns health ratio, in percentage form, capped to 100
     */
    getHealthRatioUi(healthType: HealthType): number | undefined;
    /**
     * Sum of all the assets i.e. token deposits, borrows, total assets in spot open orders, (perps positions is todo) in terms of quote value.
     * @returns equity, in native quote
     */
    getEquity(): I80F48 | undefined;
    /**
     * The amount of native quote you could withdraw against your existing assets.
     * @returns collateral value, in native quote
     */
    getCollateralValue(): I80F48 | undefined;
    /**
     * Sum of all positive assets.
     * @returns assets, in native quote
     */
    getAssetsValue(healthType: HealthType): I80F48 | undefined;
    /**
     * Sum of all negative assets.
     * @returns liabs, in native quote
     */
    getLiabsValue(healthType: HealthType): I80F48 | undefined;
    /**
     * The amount of given native token you can withdraw including borrows, considering all existing assets as collateral.
     * @returns amount of given native token you can borrow, considering all existing assets as collateral, in native token
     */
    getMaxWithdrawWithBorrowForToken(group: Group, mintPk: PublicKey): I80F48 | undefined;
    getMaxWithdrawWithBorrowForTokenUi(group: Group, mintPk: PublicKey): number | undefined;
    /**
     * The max amount of given source native token you can swap to a target token.
     * note: slippageAndFeesFactor is a normalized number, <1,
     *  e.g. a slippage of 5% and some fees which are 1%, then slippageAndFeesFactor = 0.94
     *  the factor is used to compute how much target can be obtained by swapping source
     * @returns max amount of given source native token you can swap to a target token, in native token
     */
    getMaxSourceForTokenSwap(group: Group, sourceMintPk: PublicKey, targetMintPk: PublicKey, slippageAndFeesFactor: number): I80F48 | undefined;
    /**
     * The max amount of given source ui token you can swap to a target token.
     * note: slippageAndFeesFactor is a normalized number, <1,
     *  e.g. a slippage of 5% and some fees which are 1%, then slippageAndFeesFactor = 0.94
     *  the factor is used to compute how much target can be obtained by swapping source
     * @returns max amount of given source ui token you can swap to a target token, in ui token
     */
    getMaxSourceUiForTokenSwap(group: Group, sourceMintPk: PublicKey, targetMintPk: PublicKey, slippageAndFeesFactor: number): number | undefined;
    /**
     * Simulates new health ratio after applying tokenChanges to the token positions.
     * Note: token changes are expected in native amounts
     *
     * e.g. useful to simulate health after a potential swap.
     * Note: health ratio is technically ∞ if liabs are 0
     * @returns health ratio, in percentage form
     */
    simHealthRatioWithTokenPositionChanges(group: Group, nativeTokenChanges: {
        nativeTokenAmount: I80F48;
        mintPk: PublicKey;
    }[], healthType?: HealthType): I80F48 | undefined;
    /**
     * Simulates new health ratio after applying tokenChanges to the token positions.
     * Note: token changes are expected in ui amounts
     *
     * e.g. useful to simulate health after a potential swap.
     * Note: health ratio is technically ∞ if liabs are 0
     * @returns health ratio, in percentage form
     */
    simHealthRatioWithTokenPositionUiChanges(group: Group, uiTokenChanges: {
        uiTokenAmount: number;
        mintPk: PublicKey;
    }[], healthType?: HealthType): number | undefined;
    loadSerum3OpenOrdersAccounts(client: MangoClient): Promise<OpenOrders[]>;
    loadSerum3OpenOrdersForMarket(client: MangoClient, group: Group, externalMarketPk: PublicKey): Promise<Order[]>;
    /**
     *
     * @param group
     * @param serum3Market
     * @returns maximum native quote which can be traded for base token given current health
     */
    getMaxQuoteForSerum3Bid(group: Group, serum3Market: Serum3Market): I80F48;
    getMaxQuoteForSerum3BidUi(group: Group, externalMarketPk: PublicKey): number;
    /**
     *
     * @param group
     * @param serum3Market
     * @returns maximum native base which can be traded for quote token given current health
     */
    getMaxBaseForSerum3Ask(group: Group, serum3Market: Serum3Market): I80F48;
    getMaxBaseForSerum3AskUi(group: Group, externalMarketPk: PublicKey): number;
    /**
     *
     * @param group
     * @param nativeQuoteAmount
     * @param serum3Market
     * @param healthType
     * @returns health ratio after a bid with nativeQuoteAmount is placed
     */
    simHealthRatioWithSerum3BidChanges(group: Group, nativeQuoteAmount: I80F48, serum3Market: Serum3Market, healthType?: HealthType): I80F48;
    simHealthRatioWithSerum3BidUiChanges(group: Group, uiQuoteAmount: number, externalMarketPk: PublicKey, healthType?: HealthType): number;
    /**
     *
     * @param group
     * @param nativeBaseAmount
     * @param serum3Market
     * @param healthType
     * @returns health ratio after an ask with nativeBaseAmount is placed
     */
    simHealthRatioWithSerum3AskChanges(group: Group, nativeBaseAmount: I80F48, serum3Market: Serum3Market, healthType?: HealthType): I80F48;
    simHealthRatioWithSerum3AskUiChanges(group: Group, uiBaseAmount: number, externalMarketPk: PublicKey, healthType?: HealthType): number;
    loadPerpOpenOrdersForMarket(client: MangoClient, group: Group, perpMarketName: string): Promise<PerpOrder[]>;
    toString(group?: Group): string;
}
export declare class TokenPosition {
    indexedPosition: I80F48;
    tokenIndex: number;
    inUseCount: number;
    static TokenIndexUnset: number;
    static from(dto: TokenPositionDto): TokenPosition;
    constructor(indexedPosition: I80F48, tokenIndex: number, inUseCount: number);
    isActive(): boolean;
    /**
     *
     * @param bank
     * @returns native balance
     */
    balance(bank: Bank): I80F48;
    /**
     *
     * @param bank
     * @returns native deposits, 0 if position has borrows
     */
    deposits(bank: Bank): I80F48;
    /**
     *
     * @param bank
     * @returns native borrows, 0 if position has deposits
     */
    borrows(bank: Bank): I80F48;
    /**
     * @param bank
     * @returns UI balance, is signed
     */
    balanceUi(bank: Bank): number;
    /**
     * @param bank
     * @returns UI deposits, 0 if position has borrows
     */
    depositsUi(bank: Bank): number;
    /**
     * @param bank
     * @returns UI borrows, 0 if position has deposits
     */
    borrowsUi(bank: Bank): number;
    toString(group?: Group, index?: number): string;
}
export declare class TokenPositionDto {
    indexedPosition: I80F48Dto;
    tokenIndex: number;
    inUseCount: number;
    reserved: number[];
    constructor(indexedPosition: I80F48Dto, tokenIndex: number, inUseCount: number, reserved: number[]);
}
export declare class Serum3Orders {
    openOrders: PublicKey;
    marketIndex: number;
    baseTokenIndex: number;
    quoteTokenIndex: number;
    static Serum3MarketIndexUnset: number;
    static from(dto: Serum3PositionDto): Serum3Orders;
    constructor(openOrders: PublicKey, marketIndex: number, baseTokenIndex: number, quoteTokenIndex: number);
    isActive(): boolean;
}
export declare class Serum3PositionDto {
    openOrders: PublicKey;
    marketIndex: number;
    baseTokenIndex: number;
    quoteTokenIndex: number;
    reserved: number[];
    constructor(openOrders: PublicKey, marketIndex: number, baseTokenIndex: number, quoteTokenIndex: number, reserved: number[]);
}
export declare class PerpPosition {
    marketIndex: number;
    basePositionLots: number;
    quotePositionNative: BN;
    bidsBaseLots: number;
    asksBaseLots: number;
    takerBaseLots: number;
    takerQuoteLots: number;
    static PerpMarketIndexUnset: number;
    static from(dto: PerpPositionDto): PerpPosition;
    constructor(marketIndex: number, basePositionLots: number, quotePositionNative: BN, bidsBaseLots: number, asksBaseLots: number, takerBaseLots: number, takerQuoteLots: number);
    isActive(): boolean;
}
export declare class PerpPositionDto {
    marketIndex: number;
    reserved: [];
    basePositionLots: BN;
    quotePositionNative: {
        val: BN;
    };
    bidsBaseLots: BN;
    asksBaseLots: BN;
    takerBaseLots: BN;
    takerQuoteLots: BN;
    constructor(marketIndex: number, reserved: [], basePositionLots: BN, quotePositionNative: {
        val: BN;
    }, bidsBaseLots: BN, asksBaseLots: BN, takerBaseLots: BN, takerQuoteLots: BN);
}
export declare class PerpOo {
    orderSide: any;
    orderMarket: 0;
    clientOrderId: number;
    orderId: BN;
    static OrderMarketUnset: number;
    static from(dto: PerpOoDto): PerpOo;
    constructor(orderSide: any, orderMarket: 0, clientOrderId: number, orderId: BN);
}
export declare class PerpOoDto {
    orderSide: any;
    orderMarket: 0;
    clientOrderId: BN;
    orderId: BN;
    constructor(orderSide: any, orderMarket: 0, clientOrderId: BN, orderId: BN);
}
export declare class HealthType {
    static maint: {
        maint: {};
    };
    static init: {
        init: {};
    };
}
export declare class MangoAccountData {
    healthCache: HealthCache;
    initHealth: I80F48;
    maintHealth: I80F48;
    equity: Equity;
    constructor(healthCache: HealthCache, initHealth: I80F48, maintHealth: I80F48, equity: Equity);
    static from(event: {
        healthCache: HealthCacheDto;
        initHealth: I80F48Dto;
        maintHealth: I80F48Dto;
        equity: {
            tokens: [{
                tokenIndex: number;
                value: I80F48Dto;
            }];
            perps: [{
                perpMarketIndex: number;
                value: I80F48Dto;
            }];
        };
        initHealthLiabs: I80F48Dto;
        tokenAssets: any;
    }): MangoAccountData;
}
export declare class Equity {
    tokens: TokenEquity[];
    perps: PerpEquity[];
    constructor(tokens: TokenEquity[], perps: PerpEquity[]);
    static from(dto: EquityDto): Equity;
}
export declare class TokenEquity {
    tokenIndex: number;
    value: I80F48;
    constructor(tokenIndex: number, value: I80F48);
}
export declare class PerpEquity {
    perpMarketIndex: number;
    value: I80F48;
    constructor(perpMarketIndex: number, value: I80F48);
}
export declare class EquityDto {
    tokens: {
        tokenIndex: number;
        value: I80F48Dto;
    }[];
    perps: {
        perpMarketIndex: number;
        value: I80F48Dto;
    }[];
}
