/// <reference types="bn.js" />
import { BN, Program, Provider } from '@project-serum/anchor';
import { Cluster, PublicKey, TransactionInstruction, TransactionSignature } from '@solana/web3.js';
import { Bank, MintInfo } from './accounts/bank';
import { Group } from './accounts/group';
import { MangoAccount, MangoAccountData } from './accounts/mangoAccount';
import { StubOracle } from './accounts/oracle';
import { PerpMarket, PerpOrderType, Side } from './accounts/perp';
import { Serum3Market, Serum3OrderType, Serum3SelfTradeBehavior, Serum3Side } from './accounts/serum3';
import { MangoV4 } from './mango_v4';
import { FlashLoanType, InterestRateParams } from './types';
declare enum AccountRetriever {
    Scanning = 0,
    Fixed = 1
}
export declare type IdsSource = 'api' | 'static' | 'get-program-accounts';
export declare class MangoClient {
    program: Program<MangoV4>;
    programId: PublicKey;
    cluster: Cluster;
    opts: {
        postSendTxCallback?: ({ txid }: {
            txid: string;
        }) => void;
        prioritizationFee?: number;
    };
    idsSource: IdsSource;
    private postSendTxCallback?;
    private prioritizationFee;
    constructor(program: Program<MangoV4>, programId: PublicKey, cluster: Cluster, opts?: {
        postSendTxCallback?: ({ txid }: {
            txid: string;
        }) => void;
        prioritizationFee?: number;
    }, idsSource?: IdsSource);
    groupCreate(groupNum: number, testing: boolean, version: number, insuranceMintPk: PublicKey): Promise<TransactionSignature>;
    groupEdit(group: Group, admin: PublicKey | undefined, fastListingAdmin: PublicKey | undefined, testing: number | undefined, version: number | undefined): Promise<TransactionSignature>;
    groupClose(group: Group): Promise<TransactionSignature>;
    getGroup(groupPk: PublicKey): Promise<Group>;
    getGroupsForCreator(creatorPk: PublicKey): Promise<Group[]>;
    getGroupForCreator(creatorPk: PublicKey, groupNum: number): Promise<Group>;
    tokenRegister(group: Group, mintPk: PublicKey, oraclePk: PublicKey, oracleConfFilter: number, tokenIndex: number, name: string, adjustmentFactor: number, util0: number, rate0: number, util1: number, rate1: number, maxRate: number, loanFeeRate: number, loanOriginationFeeRate: number, maintAssetWeight: number, initAssetWeight: number, maintLiabWeight: number, initLiabWeight: number, liquidationFee: number): Promise<TransactionSignature>;
    tokenRegisterTrustless(group: Group, mintPk: PublicKey, oraclePk: PublicKey, tokenIndex: number, name: string): Promise<TransactionSignature>;
    tokenEdit(group: Group, mintPk: PublicKey, oracle: PublicKey | null, oracleConfFilter: number | null, groupInsuranceFund: boolean | null, interestRateParams: InterestRateParams | null, loanFeeRate: number | null, loanOriginationFeeRate: number | null, maintAssetWeight: number | null, initAssetWeight: number | null, maintLiabWeight: number | null, initLiabWeight: number | null, liquidationFee: number | null): Promise<TransactionSignature>;
    tokenDeregister(group: Group, mintPk: PublicKey): Promise<TransactionSignature>;
    getBanksForGroup(group: Group): Promise<Bank[]>;
    getMintInfosForGroup(group: Group): Promise<MintInfo[]>;
    getMintInfoForTokenIndex(group: Group, tokenIndex: number): Promise<MintInfo[]>;
    stubOracleCreate(group: Group, mintPk: PublicKey, price: number): Promise<TransactionSignature>;
    stubOracleClose(group: Group, oracle: PublicKey): Promise<TransactionSignature>;
    stubOracleSet(group: Group, oraclePk: PublicKey, price: number): Promise<TransactionSignature>;
    getStubOracle(group: Group, mintPk?: PublicKey): Promise<StubOracle[]>;
    getOrCreateMangoAccount(group: Group, ownerPk: PublicKey, accountNumber?: number, name?: string): Promise<MangoAccount | undefined>;
    createMangoAccount(group: Group, accountNumber?: number, name?: string): Promise<TransactionSignature>;
    expandMangoAccount(group: Group, account: MangoAccount, tokenCount: number, serum3Count: number, perpCount: number, perpOoCount: number): Promise<TransactionSignature>;
    editMangoAccount(group: Group, mangoAccount: MangoAccount, name?: string, delegate?: PublicKey): Promise<TransactionSignature>;
    getMangoAccount(mangoAccount: MangoAccount): Promise<MangoAccount>;
    getMangoAccountWithSlot(mangoAccountPk: PublicKey): Promise<{
        slot: number;
        value: MangoAccount;
    } | undefined>;
    getMangoAccountForOwner(group: Group, ownerPk: PublicKey, accountNumber: number): Promise<MangoAccount | undefined>;
    getMangoAccountsForOwner(group: Group, ownerPk: PublicKey): Promise<MangoAccount[]>;
    getAllMangoAccounts(group: Group): Promise<MangoAccount[]>;
    closeMangoAccount(group: Group, mangoAccount: MangoAccount): Promise<TransactionSignature>;
    computeAccountData(group: Group, mangoAccount: MangoAccount): Promise<MangoAccountData | undefined>;
    tokenDeposit(group: Group, mangoAccount: MangoAccount, mintPk: PublicKey, amount: number): Promise<TransactionSignature>;
    tokenDepositNative(group: Group, mangoAccount: MangoAccount, mintPk: PublicKey, nativeAmount: number): Promise<string>;
    tokenWithdraw(group: Group, mangoAccount: MangoAccount, mintPk: PublicKey, amount: number, allowBorrow: boolean): Promise<TransactionSignature>;
    tokenWithdrawNative(group: Group, mangoAccount: MangoAccount, mintPk: PublicKey, nativeAmount: number, allowBorrow: boolean): Promise<TransactionSignature>;
    serum3RegisterMarket(group: Group, serum3MarketExternalPk: PublicKey, baseBank: Bank, quoteBank: Bank, marketIndex: number, name: string): Promise<TransactionSignature>;
    serum3deregisterMarket(group: Group, externalMarketPk: PublicKey): Promise<TransactionSignature>;
    serum3GetMarkets(group: Group, baseTokenIndex?: number, quoteTokenIndex?: number): Promise<Serum3Market[]>;
    serum3CreateOpenOrders(group: Group, mangoAccount: MangoAccount, externalMarketPk: PublicKey): Promise<TransactionSignature>;
    serum3CloseOpenOrders(group: Group, mangoAccount: MangoAccount, externalMarketPk: PublicKey): Promise<TransactionSignature>;
    serum3PlaceOrder(group: Group, mangoAccount: MangoAccount, externalMarketPk: PublicKey, side: Serum3Side, price: number, size: number, selfTradeBehavior: Serum3SelfTradeBehavior, orderType: Serum3OrderType, clientOrderId: number, limit: number): Promise<string>;
    serum3CancelAllorders(group: Group, mangoAccount: MangoAccount, externalMarketPk: PublicKey, limit: number): Promise<string>;
    serum3SettleFunds(group: Group, mangoAccount: MangoAccount, externalMarketPk: PublicKey): Promise<TransactionSignature>;
    serum3CancelOrder(group: Group, mangoAccount: MangoAccount, externalMarketPk: PublicKey, side: Serum3Side, orderId: BN): Promise<TransactionSignature>;
    perpCreateMarket(group: Group, oraclePk: PublicKey, perpMarketIndex: number, name: string, oracleConfFilter: number, baseDecimals: number, quoteTokenIndex: number, quoteLotSize: number, baseLotSize: number, maintAssetWeight: number, initAssetWeight: number, maintLiabWeight: number, initLiabWeight: number, liquidationFee: number, makerFee: number, takerFee: number, minFunding: number, maxFunding: number, impactQuantity: number, groupInsuranceFund: boolean, trustedMarket: boolean): Promise<TransactionSignature>;
    perpEditMarket(group: Group, perpMarketName: string, oracle: PublicKey, oracleConfFilter: number, baseDecimals: number, maintAssetWeight: number, initAssetWeight: number, maintLiabWeight: number, initLiabWeight: number, liquidationFee: number, makerFee: number, takerFee: number, minFunding: number, maxFunding: number, impactQuantity: number, groupInsuranceFund: boolean, trustedMarket: boolean): Promise<TransactionSignature>;
    perpCloseMarket(group: Group, perpMarketName: string): Promise<TransactionSignature>;
    perpGetMarkets(group: Group): Promise<PerpMarket[]>;
    perpPlaceOrder(group: Group, mangoAccount: MangoAccount, perpMarketName: string, side: Side, price: number, quantity: number, maxQuoteQuantity: number, clientOrderId: number, orderType: PerpOrderType, expiryTimestamp: number, limit: number): Promise<TransactionSignature>;
    perpCancelAllOrders(group: Group, mangoAccount: MangoAccount, perpMarketName: string, limit: number): Promise<TransactionSignature>;
    marginTrade({ group, mangoAccount, inputMintPk, amountIn, outputMintPk, userDefinedInstructions, flashLoanType, }: {
        group: Group;
        mangoAccount: MangoAccount;
        inputMintPk: PublicKey;
        amountIn: number;
        outputMintPk: PublicKey;
        userDefinedInstructions: TransactionInstruction[];
        flashLoanType: FlashLoanType;
    }): Promise<TransactionSignature>;
    updateIndexAndRate(group: Group, mintPk: PublicKey): Promise<void>;
    liqTokenWithToken(group: Group, liqor: MangoAccount, liqee: MangoAccount, assetMintPk: PublicKey, liabMintPk: PublicKey, maxLiabTransfer: number): Promise<void>;
    static connect(provider: Provider, cluster: Cluster, programId: PublicKey, opts?: any, getIdsFromApi?: IdsSource): MangoClient;
    static connectForGroupName(provider: Provider, groupName: string): MangoClient;
    buildHealthRemainingAccounts(retriever: AccountRetriever, group: Group, mangoAccounts: MangoAccount[], banks: Bank[], perpMarkets: PerpMarket[]): PublicKey[];
    buildFixedAccountRetrieverHealthAccounts(group: Group, mangoAccount: MangoAccount, banks: Bank[], perpMarkets: PerpMarket[]): PublicKey[];
    buildScanningAccountRetrieverHealthAccounts(group: Group, mangoAccounts: MangoAccount[], banks: Bank[], perpMarkets: PerpMarket[]): PublicKey[];
}
export {};
